/*
 *  Copyright 2019 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.core.util.date;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.TemporalUnit;

/**
 * This represents an advanced {@link LocalDateTime}, allowing to be a static one, 
 * or a relative one to the moment the {@link #resolveDateTime} method is called.
 * <br>A single instance of {@link AdaptableDate} can be {@link #resolveDateTime() resolved} mutliple times, without being a problem at all, it just can lead to different results.
 * <br>
 * <br>You can get an {@link AdaptableDate} with:
 * <ul>
 * <li>{@link #fromDateTime} to have a static one, always resolved as the same given {@link LocalDateTime}</li>
 * <li>{@link #fromDate} to have a static one, always resolved as the same given {@link LocalDate} (at the start of the day)</li>
 * <li> {@link #now} to have one resolved as the current {@link LocalDateTime} when {@link #resolveDateTime} is called</li>
 * <li> {@link #future} to have one resolved with the given amount of given {@link TemporalUnit} added to the current {@link LocalDateTime} when {@link #resolveDateTime} is called</li>
 * <li> {@link #past} to have one resolved with the given amount of given {@link TemporalUnit} subtracted to the current {@link LocalDateTime} when {@link #resolveDateTime} is called</li>
 * </ul>
 */
public final class AdaptableDate
{
    private static enum AdaptableDateType
    {
        STATIC,
        RELATIVE_TO_NOW,
    }
    
    private static enum OffsetType
    {
        NOW,
        PAST,
        FUTURE,
    }
    
    private static final AdaptableDate __NOW = new AdaptableDate(OffsetType.NOW, 0, null);
    
    private final AdaptableDateType _type;
    private final LocalDateTime _staticValue;
    private final TemporalUnit _relativeUnit;
    private final long _relativeAmount;
    private final OffsetType _relativeOffsetType;
    
    private AdaptableDate(LocalDateTime date)
    {
        _type = AdaptableDateType.STATIC;
        _staticValue = date;
        _relativeUnit = null;
        _relativeAmount = 0;
        _relativeOffsetType = null;
    }
    
    private AdaptableDate(OffsetType offsetType, long amount, TemporalUnit unit)
    {
        _type = AdaptableDateType.RELATIVE_TO_NOW;
        _staticValue = null;
        _relativeUnit = unit;
        _relativeAmount = amount;
        _relativeOffsetType = offsetType;
    }
    
    /**
     * Gets a static {@link AdaptableDate}, always resolved as the given {@link LocalDateTime}
     * @param dateTime The {@link LocalDateTime}
     * @return The {@link AdaptableDate}
     */
    public static AdaptableDate fromDateTime(LocalDateTime dateTime)
    {
        return new AdaptableDate(dateTime);
    }
    
    /**
     * Gets a static {@link AdaptableDate}, always resolved as the given {@link LocalDate}
     * @param date The {@link LocalDate}
     * @return The {@link AdaptableDate}
     */
    public static AdaptableDate fromDate(LocalDate date)
    {
        return new AdaptableDate(date.atStartOfDay());
    }
    
    /**
     * Gets an {@link AdaptableDate}, resolved as the current {@link LocalDateTime} at the moment of the {@link #resolveDateTime() call}
     * @return The {@link AdaptableDate}
     */
    public static AdaptableDate now()
    {
        return __NOW;
    }
    
    /**
     * Gets an {@link AdaptableDate}, resolved as the current {@link LocalDateTime} at the moment of the {@link #resolveDateTime() call}, 
     * minus the given amount of the given {@link TemporalUnit}, so as to get a {@link LocalDateTime} in the past
     * @param amount The amount to subtract
     * @param unit The {@link TemporalUnit} of the amount to subtract
     * @return The {@link AdaptableDate}
     */
    public static AdaptableDate past(long amount, TemporalUnit unit)
    {
        return new AdaptableDate(OffsetType.PAST, amount, unit);
    }
    
    /**
     * Gets an {@link AdaptableDate}, resolved as the current {@link LocalDateTime} at the moment of the {@link #resolveDateTime() call}, 
     * plus the given amount of the given {@link TemporalUnit}, so as to get a {@link LocalDateTime} in the future
     * @param amount The amount to add
     * @param unit The {@link TemporalUnit} of the amount to add
     * @return The {@link AdaptableDate}
     */
    public static AdaptableDate future(long amount, TemporalUnit unit)
    {
        return new AdaptableDate(OffsetType.FUTURE, amount, unit);
    }
    
    /**
     * Resolves this {@link AdaptableDate}
     * @return The resolved {@link LocalDateTime}
     */
    public LocalDateTime resolveDateTime()
    {
        switch (_type)
        {
            case RELATIVE_TO_NOW:
                LocalDateTime now = LocalDateTime.now();
                LocalDateTime computed;
                switch (_relativeOffsetType)
                {
                    case NOW:
                        computed = now;
                        break;
                    case PAST:
                        computed = now.minus(_relativeAmount, _relativeUnit);
                        break;
                    case FUTURE:
                    default:
                        computed = now.plus(_relativeAmount, _relativeUnit);
                        break;
                }
                return computed;
            
            case STATIC:
            default:
                return _staticValue;
        }
    }
    
    /**
     * Resolves this {@link AdaptableDate}
     * @return The resolved {@link LocalDate}
     */
    public LocalDate resolveDate()
    {
        return resolveDateTime()
                .toLocalDate();
    }
    
    @Override
    public String toString()
    {
        switch (_type)
        {
            case STATIC:
                return _staticValue.toString();
            case RELATIVE_TO_NOW:
            default:
                switch (_relativeOffsetType)
                {
                    case NOW:
                        return "Now";
                    case PAST:
                        return _relativeAmount + " " + _relativeUnit + " ago";
                    case FUTURE:
                    default:
                        return "In " + _relativeAmount + " " + _relativeUnit;
                }
        }
    }
}

