/*
 *  Copyright 2019 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.core.util.date;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;

import org.apache.commons.lang3.StringUtils;

/**
 * Class for parsing a {@link String} and converting it to an {@link AdaptableDate}.
 * <br>See documentation of {@link #parse} method.
 */
public final class AdaptableDateParser
{
    private static final String __SPECIAL_CHAR = "$";
    private static final String __CURRENT = "current";
    private static final String __AGO = "ago" + __SPECIAL_CHAR;
    private static final String __IN = "in" + __SPECIAL_CHAR;
    
    private AdaptableDateParser()
    { }
    
    /**
     * Parses the given {@link String} as an {@link AdaptableDate}
     * <br>It accepts the following syntax:
     * <ul>
     * <li><code>2019-01-01T09:30:00+01:00</code> to have a {@link AdaptableDate#fromDateTime static} date (based on the {@link DateTimeFormatter#ISO_DATE_TIME} format)</li>
     * <li><code>$current</code> to have the {@link AdaptableDate#now current} date</li>
     * <li><code>$ago$2$Minutes</code> to have a date in the {@link AdaptableDate#past past}</li>
     * <li><code>$in$3$Weeks</code> to have a date in the {@link AdaptableDate#future future}</li>
     * </ul>
     * @param value The string value to parse
     * @return The {@link AdaptableDate}
     * @throws IllegalArgumentException if the string value cannot be parsed as an {@link AdaptableDate}
     */
    public static final AdaptableDate parse(String value)
    {
        if (value.startsWith(__SPECIAL_CHAR))
        {
            String relativeDate = StringUtils.substringAfter(value, __SPECIAL_CHAR);
            if (__CURRENT.equals(relativeDate))
            {
                return AdaptableDate.now();
            }
            else if (relativeDate.startsWith(__AGO))
            {
                String amountAndUnit = StringUtils.substringAfter(relativeDate, __AGO);
                String unitStr = StringUtils.substringAfterLast(amountAndUnit, __SPECIAL_CHAR);
                Long amount = Long.valueOf(StringUtils.substringBeforeLast(amountAndUnit, __SPECIAL_CHAR));
                return AdaptableDate.past(amount, ChronoUnit.valueOf(unitStr));
            }
            else if (relativeDate.startsWith(__IN))
            {
                String amountAndUnit = StringUtils.substringAfter(relativeDate, __IN);
                String unitStr = StringUtils.substringAfterLast(amountAndUnit, __SPECIAL_CHAR);
                Long amount = Long.valueOf(StringUtils.substringBeforeLast(amountAndUnit, __SPECIAL_CHAR));
                return AdaptableDate.future(amount, ChronoUnit.valueOf(unitStr));
            }
            else
            {
                throw new IllegalArgumentException("'" + value + "' cannot be parsed.");
            }
        }
        else if (value.indexOf('T') > 0)
        {
            // date + time
            return AdaptableDate.fromDateTime(LocalDateTime.parse(value, DateTimeFormatter.ISO_DATE_TIME));
        }
        else
        {
            // date only
            return AdaptableDate.fromDate(LocalDate.parse(value, DateTimeFormatter.ISO_LOCAL_DATE));
        }
    }
}

